@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/pathnames.h,v 3.2 1991/10/12 04:23:51 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/pathnames.h,v 3.4 1991/12/14 20:45:46 christos Exp $ */
 /*
  * pathnames.h: Location of things to find
  */
@@ -48,15 +48,15 @@
 #define _PATH_USRBSD		"/usr/bsd"
 #define _PATH_BIN		"/bin"
 
-#if defined(convex) || defined(__convex__)
+#if defined(convex) || defined(__convex__) || defined(stellar)
 # define _PATH_DOTLOGIN		"/etc/login"
 # define _PATH_DOTLOGOUT	"/etc/logout"
 # define _PATH_DOTCSHRC		"/etc/cshrc"
-#endif /* convex || __convex__ */
+#endif /* convex || __convex__ || stellar */
 
-#if defined(sgi) || defined(OREO)
+#if defined(sgi) || defined(OREO) || defined(cray)
 # define _PATH_DOTLOGIN		"/etc/cshrc"
-#endif /* sgi || OREO */
+#endif /* sgi || OREO || cray */
 
 #if defined(NeXT)
 # define _PATH_DOTLOGIN		"/etc/login.std"
