@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/tc.os.c,v 3.10 1991/10/21 17:24:49 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/tc.os.c,v 3.13 1991/12/19 22:34:14 christos Exp $ */
 /*
  * tc.os.c: OS Dependent builtin functions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.os.c,v 3.10 1991/10/21 17:24:49 christos Exp $")
+RCSID("$Id: tc.os.c,v 3.13 1991/12/19 22:34:14 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -79,7 +79,7 @@ dosetpath(arglist, c)
     struct command *c;
 {
     extern char *getenv();
-
+    sigmask_t omask;
     Char  **pathvars, **cmdargs;
     Char  **paths;
     char  **spaths, **cpaths, **cmds;
@@ -87,6 +87,18 @@ dosetpath(arglist, c)
     unsigned int npaths, ncmds;
     int     i, sysflag;
 
+    omask = sigsetmask(sigmask(SIGINT));
+
+    /*
+     * setpath(3) uses stdio and we want 0, 1, 2 to work...
+     */
+    if (!didfds) {
+	(void) dcopy(SHIN, 0);
+	(void) dcopy(SHOUT, 1);
+	(void) dcopy(SHDIAG, 2);
+	didfds = 1;
+    }
+
     for (i = 1; arglist[i] && (arglist[i][0] != '-'); i++);
     npaths = i - 1;
 
@@ -131,12 +143,13 @@ dosetpath(arglist, c)
     for (i = 0; i < ncmds; i++) {
 	Char   *val = globone(cmdargs[i], G_ERROR);
 
-	if (val == NOSTR)
+	if (val == NULL)
 	    goto abortpath;
 	cmds[i] = xmalloc(Strlen(val) + 1);
 	(void) strcpy(cmds[i], short2str(val));
     }
 
+
     if (setpath(cpaths, cmds, LOCALSYSPATH, sysflag, 1) < 0) {
 abortpath:
 	if (spaths) {
@@ -164,6 +177,8 @@ abortpath:
 	    paths[i] = SAVE(cpaths[i]);
 	    xfree((ptr_t) cpaths[i]);
 	}
+	(void) sigsetmask(omask);
+	donefds();
 	return;
     }
 
@@ -182,6 +197,8 @@ abortpath:
 	    *--val = '=';
 	}
     }
+    (void) sigsetmask(omask);
+    donefds();
 }
 #endif /* MACH */
 
@@ -716,9 +733,6 @@ osinit()
     }
 #endif /* aiws */
 
-#ifdef hpux
-    (void) sigspace(4192);
-#endif	/* hpux */
 #ifdef titan
     end = sbrk(0);
 #endif	/* titan */
@@ -992,7 +1006,7 @@ static char *
 apperr(st)
     status_$t *st;
 {
-    static char buf[BUFSIZ];
+    static char buf[BUFSIZE];
     short e_subl, e_modl, e_codel;
     error_$string_t e_sub, e_mod, e_code;
 
