@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/tc.prompt.c,v 3.5 1991/10/12 04:23:51 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/tc.prompt.c,v 3.9 1991/12/19 22:34:14 christos Exp $ */
 /*
  * tc.prompt.c: Prompt printing stuff
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.prompt.c,v 3.5 1991/10/12 04:23:51 christos Exp $")
+RCSID("$Id: tc.prompt.c,v 3.9 1991/12/19 22:34:14 christos Exp $")
 
 #include "ed.h"
 
@@ -56,20 +56,21 @@ printprompt(promptno, str)
     int     promptno;
     Char   *str;
 {
-    Char   *p, *z, *q;
+    Char   *p, *z, *q, *ep, *cp;
     register Char attributes = 0;
     static int print_prompt_did_ding = 0;
     register char *cz;
     struct tm *t;
     time_t  lclock;
-    Char    buff[BUFSIZ];
-    Char   *cp;
+    Char    buff[BUFSIZE];
+    static  Char *ocp = NULL, *ostr = NULL;
 
     (void) time(&lclock);
     t = localtime(&lclock);
 
     PromptBuf[0] = '\0';
     p = PromptBuf;
+    ep = &PromptBuf[2*INBUFSIZE - 2];
     switch (promptno) {
     default:
     case 0:
@@ -81,16 +82,32 @@ printprompt(promptno, str)
     case 2:
 	cp = value(STRprompt3);
 	break;
+    case 3:
+	if (ocp != NULL) {
+	    cp = ocp;
+	    str = ostr;
+	}
+	else 
+	    cp = value(STRprompt);
+	break;
+    }
+    if (promptno < 2) {
+	ocp = cp;
+	ostr = str;
     }
 
     for (; *cp; cp++) {
+	if (p >= ep)
+	    break;
 	if (*cp == '%') {
 	    cp++;
 	    switch (*cp) {
 	    case 'R':
 		if (str != NULL)
-		    while (*str)
+		    while (*str) {
 			*p++ = attributes | *str++;
+			if (p >= ep) break;
+		    }
 		break;
 	    case '#':
 		*p++ = attributes | ((uid == 0) ? '#' : '>');
@@ -98,8 +115,10 @@ printprompt(promptno, str)
 	    case '!':
 	    case 'h':
 		Itoa(eventno + 1, buff);
-		for (z = buff; *z; z++)
+		for (z = buff; *z; z++) {
 		    *p++ = attributes | *z;
+		    if (p >= ep) break;
+		}
 		break;
 	    case 'T':		/* 24 hour format	 */
 	    case '@':
@@ -108,6 +127,8 @@ printprompt(promptno, str)
 		    char    ampm = 'a';
 		    int     hr = t->tm_hour;
 
+		    if (p >= ep - 10) break;
+
 		    /* addition by Hans J. Albertsson */
 		    /* and another adapted from Justin Bur */
 		    if (adrof(STRampm) || *cp != 'T') {
@@ -127,7 +148,7 @@ printprompt(promptno, str)
 			    print_prompt_did_ding = 0;
 			Itoa(hr, buff);
 			*p++ = attributes | buff[0];
-			if (buff[1])
+			if (buff[1]) 
 			    *p++ = attributes | buff[1];
 			*p++ = attributes | ':';
 			Itoa(t->tm_min, buff);
@@ -162,14 +183,18 @@ printprompt(promptno, str)
 		 * derefrence that NULL (if HOST is not set)...
 		 */
 		if ((cz = getenv("HOST")) != NULL)
-		    while (*cz)
+		    while (*cz) {
+			if (p >= ep) break;
 			*p++ = attributes | *cz++;
+		    }
 		break;
 
 	    case 'm':
 		if ((cz = getenv("HOST")) != NULL)
-		    while (*cz && *cz != '.')
+		    while (*cz && *cz != '.') {
+			if (p >= ep) break;
 			*p++ = attributes | *cz++;
+		    }
 		break;
 
 	    case '~':		/* show ~ whenever possible - a la dirs */
@@ -188,16 +213,21 @@ printprompt(promptno, str)
 		    }
 		    if (olduser) {
 			*p++ = attributes | '~';
-			for (q = olduser; *q; *p++ = attributes | *q++);
-			for (z = oldpath; *z; *p++ = attributes | *z++);
+			if (p >= ep) break;
+			for (q = olduser; *q; *p++ = attributes | *q++)
+			    if (p >= ep) break;
+			for (z = oldpath; *z; *p++ = attributes | *z++)
+			    if (p >= ep) break;
 			break;
 		    }
 		}
 		/* fall through if ~ not matched */
 	    case '/':
 		if (z = value(STRcwd)) {
-		    while (*z)
+		    while (*z) {
 			*p++ = attributes | *z++;
+			if (p >= ep) break;
+		    }
 		}
 		break;
 	    case '.':
@@ -240,27 +270,36 @@ printprompt(promptno, str)
 			}
 			if (*z == '/' && z != q)
 			    z++;
-			while (*z)
+			while (*z) {
 			    *p++ = attributes | *z++;
+			    if (p >= ep) break;
+			}
 		    }
 		}
 		break;
 	    case 'n':
 		if (z = value(STRuser))
-		    while (*z)
+		    while (*z) {
 			*p++ = attributes | *z++;
+			if (p >= ep) break;
+		    }
 		break;
 	    case 'l':
 		if (z = value(STRtty))
-		    while (*z)
+		    while (*z) {
 			*p++ = attributes | *z++;
+			if (p >= ep) break;
+		    }
 		break;
 	    case 'd':
-		for (cz = day_list[t->tm_wday]; *cz;)
+		for (cz = day_list[t->tm_wday]; *cz;) {
 		    *p++ = attributes | *cz++;
+		    if (p >= ep) break;
+		}
 		break;
 	    case 'D':
 		Itoa(t->tm_mday, buff);
+		if (p >= ep - 3) break;
 		if (buff[1]) {
 		    *p++ = attributes | buff[0];
 		    *p++ = attributes | buff[1];
@@ -271,10 +310,12 @@ printprompt(promptno, str)
 		}
 		break;
 	    case 'w':
-		for (cz = month_list[t->tm_mon]; *cz;)
+		if (p >= ep - 20) break;
+		for (cz = month_list[t->tm_mon]; *cz;) 
 		    *p++ = attributes | *cz++;
 		break;
 	    case 'W':
+		if (p >= ep - 3) break;
 		Itoa(t->tm_mon + 1, buff);
 		if (buff[1]) {
 		    *p++ = attributes | buff[0];
@@ -286,6 +327,7 @@ printprompt(promptno, str)
 		}
 		break;
 	    case 'y':
+		if (p >= ep - 3) break;
 		Itoa(t->tm_year, buff);
 		if (buff[1]) {
 		    *p++ = attributes | buff[0];
@@ -295,7 +337,9 @@ printprompt(promptno, str)
 		    *p++ = attributes | '0';
 		    *p++ = attributes | buff[0];
 		}
+		break;
 	    case 'Y':
+		if (p >= ep - 5) break;
 		Itoa(t->tm_year + 1900, buff);
 		*p++ = attributes | buff[0];
 		*p++ = attributes | buff[1];
@@ -325,8 +369,10 @@ printprompt(promptno, str)
 		break;
 	    case '?':
 		if (z = value(STRstatus))
-		    while (*z)
+		    while (*z) {
 			*p++ = attributes | *z++;
+			if (p >= ep) break;
+		    }
 		break;
 	    case '%':
 		*p++ = attributes | '%';
@@ -346,6 +392,7 @@ printprompt(promptno, str)
 		attributes &= ~LITERAL;
 		break;
 	    default:
+		if (p >= ep - 3) break;
 		*p++ = attributes | '%';
 		*p++ = attributes | *cp;
 		break;
@@ -356,8 +403,10 @@ printprompt(promptno, str)
 	}
 	else if (*cp == '!') {	/* EGS: handle '!'s in prompts */
 	    Itoa(eventno + 1, buff);
-	    for (z = buff; *z; z++)
+	    for (z = buff; *z; z++) {
 		*p++ = attributes | *z;
+		if (p >= ep) break;
+	    }
 	}
 	else {
 	    *p++ = attributes | *cp;	/* normal character */
