@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/tc.wait.h,v 3.1 1991/10/12 04:23:51 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/tc.wait.h,v 3.3 1991/11/26 04:28:26 christos Exp $ */
 /*
  * tc.wait.h: <sys/wait.h> for machines that don't have it or have it and
  *	      is incorrect.
@@ -67,6 +67,11 @@
 # include <sys/wait.h>
 #endif /* SVID == 0 */
 
+#ifdef _MINIX
+# undef NEEDwait
+# include "mi.wait.h"
+#endif /* _MINIX */
+
 #ifdef NEEDwait
 /*
  *	This wait is for big-endians and little endians
